<?php
namespace SM\Controller;

use SM\Controller\SMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use SM\Service\Sm00006000ServicosService;
use SM\Traits\TResponse;

class Sm00006000ServicosController extends SMCrud
{

    use TResponse;

    protected $ServicoService;

    public function __construct($crudService, $environment, Sm00006000ServicosService $ServicoService)
    {
            parent::__construct($crudService, $environment);
            $this->ServicoService = $ServicoService;
    }

    public function save(Request $request, Response $response)
    {
        try {
            $nrorg = $this->environment->getNrOrg();
            
            foreach ($request->getParameters() as &$row) {
                $row["NRRELACSERVHABILI"] = $this->newCode->getCodeByTableName(" FCT_RELACSERVHABILI", $nrorg, 1, 10, true);
            }
            unset($row);
            
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getInformacoesGridPrincipal()
    {
        try {
            $data = $this->ServicoService->getInformacoesGridPrincipal();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getProdutoFaturamento()
    {
        try {
            $data = $this->ServicoService->getProdutoFaturamento();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
    
    public function insertServico(Request $request)
    {
        try {
            $parametros = $request->getParameters();

            $idativo = $parametros['IDATIVO'];
            $nmservorca = $parametros['NMSERVORCA'];
            $cdprodfatserv = $parametros['CDPRODFATSERV'];
            $dsservorca = $parametros['DSSERVORCA'];
            $dsservorcanf = $parametros['DSSERVORCANF'];
            $dsservorcabol = $parametros['DSSERVORCABOL'];
            $cdservorca = $parametros['CDSERVORCA'];

            $data = $this->ServicoService->insertServico($idativo, $nmservorca, $cdprodfatserv, $dsservorca, $dsservorcanf, $dsservorcabol, $cdservorca);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deletarServicos(Request $request)
    {
        try {
            $codigos = $request->getParameters()["CDSERVORCA"];
            $this->ServicoService->deletarServicos($codigos);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function validacao(Request $request, Response $response)
    {
        try {
            $row = $request->getParameters();
            $nrorg = $this->environment->getNrOrg();
            $data = $this->ServicoService->validacao($row["CDSERVORCA"], $row["CDHABILITSERV"], $nrorg);
            $this->sendResponse($data, 200);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findHabilidades(Request $request)
    {
        try {
            $data = $this->ServicoService->findHabilidades($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function removeRelacaoHabilidade(Request $request)
    {
        try {
            $data = $this->ServicoService->removeRelacaoHabilidade($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertRelacaoHabilidade(Request $request)
    {
        try {
            $row = $request->getParameters();
            $nrorg = $this->environment->getNrOrg();
            $data = $this->ServicoService->insertRelacaoHabilidade($row["CDSERVORCA"], $row["CDHABILITSERV"], $nrorg);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function validaServicoRelacionado(Request $request)
    {
        try {
            $row = $request->getParameters();
            $servOrcaRelac = $row['CDSERVORCARELAC'];
            $servOrca = $row['CDSERVORCA'];
            $tipoMedida = $row['CDTPMEDIDASERV'];
            
            $ValidaServRelac = $this->ServicoService->validaServicoRelacionado($servOrcaRelac, $servOrca, $tipoMedida);
            
            $data =  array('VALIDA'=>$ValidaServRelac);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getRelacServico(Request $request)
    {
        try {
            $data = $this->ServicoService->getRelacServico($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertRelacServico(Request $request)
    {
        try {
            $data = $this->ServicoService->insertRelacServico($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteRelacServico(Request $request)
    {
        try {
            $data = $this->ServicoService->deleteRelacServico($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
