<?php

namespace SM\Service;

use Doctrine\ORM\EntityManager;
use SM\Factory\SmNextDocumentoCdaFactory;
use Teknisa\Libs\Util\Environment;
use SM\Util\Utilities;
use SM\Util\PHPWord\SMTemplateProcessor;

class SmNextDocumentoCdaService
{
    protected $connection;
    protected $documentoFactory;
    protected $Environment;
    protected $fileServerUrl;
    protected $smdocumentos;
    protected $fileServerApiKeys;
    protected $tempDirFS;
    
    private $tempDir;
    
    public function __construct(EntityManager $entityManager, SmNextDocumentoCdaFactory $documentoFactory, Environment $Environment, $fileServerUrl, $fileServerApiKeys)
    {
        $this->connection        = $entityManager->getConnection();
        $this->documentoFactory  = $documentoFactory;
        $this->Environment       = $Environment;
        $this->fileServerUrl     = $fileServerUrl;
        $this->fileServerApiKeys = $fileServerApiKeys;
        
        $this->tempDir = sys_get_temp_dir();
        $this->tempDirFS = sys_get_temp_dir();
    }
    
    /**
     * Gera o documento da proposta.
     */
    public function gerarDocumentoCDA($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrorg();

            $nomeArquivoNovo = null;
            $nomeArquivo = array();
            $ds = DIRECTORY_SEPARATOR;  // Essa linha somente cria um separador de diretório (/) para compor os caminhos

            $dadosCDA = $this->getDadosCDA($parametros['CDA']); //Esse select, trás os dados da CDA, para serem substituídos no documento
            $participantes = $this->getParticipantesCda($parametros['CDA']);
            $horasNFaturadas = $this->getHorasNaoFaturadas($parametros['CDA']);

            $doc = $this->getDocumentoCDA($parametros['CDA'], $parametros['IDIOMA']); // Trás o documento da CDA, a ser gerado e substituído os menemonicos.
            
            $microtime = microtime(true);
            $microtime = str_replace(array(".", "&"), "_", $microtime);
            
            if ($doc["DSTIPO"] == "application/pdf") {
                if (!in_array($doc["DSCAMINHOARQ"], $nomeArquivo)) {
                    $fileName = $microtime . "_" . $doc["NMARQUIVO"];
                    $caminhoFisicoArquivo = $this->downloadDocFileServer($this->fileServerUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);

                    $nomeArquivo = array(
                        "CAMINHO"        => "/downloadArquivo?requestType=Row&row[FILENAME]=" . urlencode(base64_encode($fileName)) . "&row[FILETYPE]=" . $doc["DSTIPO"],
                        "DADOSDOCUMENTO" => array(
                            "NMARQUIVO"    => $doc["NMARQUIVO"],
                            "DSCAMINHOARQ" => $caminhoFisicoArquivo,
                            "TYPE"         => $doc["DSTIPO"]
                        )
                    );
                }
            } else {
                if (!empty($doc["DSCAMINHOARQ"])) {   // Pega o caminho do arquivo no FileServer
                    $fileName = $microtime . "_" . $doc["NMARQUIVO"];   // Pega o nome do arquivo
                    $nomeArquivoDocx = $this->downloadDocFileServer($this->fileServerUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);  // Faz o Download do arquivo do File Server.

                    if (is_file($nomeArquivoDocx)) { // Se ele achou o arquivo, começa a  substituir os mnemônicos
                        $templateProcessor = new SMTemplateProcessor($nomeArquivoDocx); // Transforma o documento anexado em um arquivo editável no PHP

                        //substitui os dados da cda no documento
                        $this->substituirDadosCDA($dadosCDA, $horasNFaturadas, $templateProcessor);
                        
                        $documento = $templateProcessor->getTempDocumentMainPart();
                    
                        // Coloca os índices nos mnemônicos dos participantes
                        $this->gerarDocumentoParteDeParticipantes($parametros['CDA'], $nrorg, $documento);

                        // substituir participantes
                        $this->substituirDadosParticipantes($participantes, $documento);
                    
                        $templateProcessor->setTempDocumentMainPart($documento);

                        $fileName = $microtime . "_" .$dadosCDA['CDCLIENTE']. "_" .$dadosCDA['CDFUNC']. "_" .$dadosCDA['CDEMPRESA']. "_" .$dadosCDA['NRSERVPREST']. "_" .$dadosCDA['DTSERVPREST'];
                        $fileName = str_replace(array(",", "&"), "_", $fileName);
                        $fileName = str_replace(array("/"), '', $fileName);
                        $nomeArquivoNovo = $this->tempDir . $ds . $fileName;

                        $templateProcessor->saveAs($nomeArquivoNovo);
                        
                        if (!is_file($nomeArquivoNovo)) {
                            throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.");
                        }
                            
                        //gerando pdf
                        $cmd = "export HOME=" .$this->tempDir. " && soffice --headless --convert-to pdf --outdir " .$this->tempDir. " \"" .$nomeArquivoNovo. "\" 2>&1";
                            
                        $resultadoCmd = shell_exec($cmd);
                        
                        if (strpos(strtolower($resultadoCmd), "error") !== false) {
                            throw new \Exception("Falha ao gerar o arquivo PDF. Provavelmente o arquivo 'docx' foi gerado com erros, sendo impossível sua leitura para a convertê-lo em PDF.<br/>Erro: ".$resultadoCmd);
                        } else {
                            if (is_file($nomeArquivoNovo)) {
                                unlink($nomeArquivoNovo);
                            }
                            
                            $fileName = $microtime . "_" .$dadosCDA['CDCLIENTE']. "_" .$dadosCDA['CDFUNC']. "_" .$dadosCDA['CDEMPRESA']. "_" .$dadosCDA['NRSERVPREST']. "_" .$dadosCDA['DTSERVPREST'];
                            $fileName = str_replace(array(",", "&"), "_", $fileName);
                            $fileName = str_replace(array("/"), '', $fileName);
                            $nomeArquivoNovo = $this->tempDir . $ds . $fileName;

                            $nomeArquivo = array(
                                    "CAMINHO"        => "/downloadArquivo?requestType=Row&row[FILENAME]=" .  urlencode(base64_encode($fileName)) . "&row[FILETYPE]=application/pdf",
                                    "DADOSDOCUMENTO" => array(
                                        "NMARQUIVO" => $fileName,
                                        "DSCAMINHOARQ" => $this->tempDir . $ds . $fileName,
                                        "TYPE"         => $doc["DSTIPO"]
                                    )
                            );

                            // if (is_file($nomeArquivoNovo)) {

                            // } else {
                            //     throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.<br/>Erro: ".$resultadoCmd);
                            // }
                        }
                    } else {
                        throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoDocx."\" não foi encontrado.");
                    }
                }
            }
                    
            if (is_file($nomeArquivoDocx)) {
                unlink($nomeArquivoDocx);
            }
            
            return [$nomeArquivo];
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Retorna o documento da CDA.
     */
    private function getDocumentoCDA($parametros, $cdidiom)
    {
        $dados = $this->documentoFactory->getDocumentoCDA(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
            "NRORG"               => $this->Environment->getNrorg(),
            "CDIDIOM"             => $cdidiom
        ));

        return $dados[0];
    }
    
    /**
     * Retorna os dados principais da CDA
     */
    private function getDadosCDA($parametros)
    {
        $dados = $this->documentoFactory->getDadosCDA(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
            "NRORG"               => $this->Environment->getNrorg(),
        ));
        return $dados[0];
    }
    
    private function getParticipantesCda($parametros)
    {
        $dados = $this->documentoFactory->getParticipantesCda(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        ));
        return $dados[0];
    }

    private function getHorasNaoFaturadas($parametros)
    {
        $dados = $this->documentoFactory->getHorasNaoFaturadas(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        ));
        return $dados[0];
    }

    /**
     * Substitui os dados da CDA no documento.
     */
    private function substituirDadosCDA($cda, $horasNFaturadas, $templateProcessor)
    {
        /**
         * Nesta função são criados quias serão os mnemônicos a serem substituidos e os valores que irão  substituir.
         * No caso, os menemônicos devem ser criados de acordo com os dados que exitem no documento, por exemplo o mnemônico [NMPARCEIRO] que será subistituído
         * pelo nome do Parceiro. Este nome do parceiro, assim como as demais informações. você receberá através de um select na função "getDadosCDA".
         * Obs: Se você sentir a necessidade de separar as informações em selects disitintos, não tem problema, somente deve seguir o mesmo modelo. */
        
        $dataAtual = date_create();

        //Dados da CDA
        $templateProcessor->setValue("[NMRAZ0SOCIAL]", $cda["NMRAZSOCCLIE"]);
        $templateProcessor->setValue("[NMFANTASIA]", $cda["NMFANTCLIE"]);
        $templateProcessor->setValue("[REPRESENTCLI]", $cda["NMINTCLIENTE"]);
        $templateProcessor->setValue("[LOCALSERVICO]", $cda["NMFANTLUSO"]);
        $templateProcessor->setValue("[PROPOSTA]", $cda["PROPOSTA"]);
        $templateProcessor->setValue("[REPRESENTTEK]", $cda["NMFUNC"]);
        $templateProcessor->setValue("[PROJETO]", $cda["NMPROJETO"]);
        $templateProcessor->setValue("[DTSERVICO]", $cda["DTSERVPREST"]);
        $templateProcessor->setValue("[TPATIVIDADE]", $cda["DTSERVPREST"]);
        $templateProcessor->setValue("[DESCRICAOCDA]", $cda["DSSERVPREST"]);
        $templateProcessor->setValue("[HRENTRADA]", $cda["DTENTRADA"]);
        $templateProcessor->setValue("[HRSAIDA]", $cda["DTSAIDA"]);
        $templateProcessor->setValue("[HRALMOCO]", $cda["HRAUSENCIAALMOCO"]);
        $templateProcessor->setValue("[HRQTSERVPREST]", $cda["QTHRTOTAL"]);
        $templateProcessor->setValue("[HRQTDESFAT]", $cda["QTHRDESLOCAMENTOFAT"]);
        $templateProcessor->setValue("[HRQTFATURADAS]", $cda["QTHRATIVIDADEFAT"]);

        //Dados das horas não faturadas
        $templateProcessor->setValue("[TAGHORA]", $horasNFaturadas["DSJUSTHORASNAOFATUCDA"]);
        $templateProcessor->setValue("[HORA]", $horasNFaturadas["QTHORASNAOFATU"]);

        $templateProcessor->setValue("[DTHOJE]", $dataAtual->format("d/m/Y"));
        $templateProcessor->setValue("[DIAAT]", $dataAtual->format("d"));
        $templateProcessor->setValue("[MESAT]", $dataAtual->format("m"));
        $templateProcessor->setValue("[ANOAT]", $dataAtual->format("Y"));
        
        $nomeMes = Utilities::getNomeMesPortugues($dataAtual);
        $templateProcessor->setValue("[NMMESAT]", $nomeMes);
    }

    /**
     * ============================================
     *   Funções do File Server
     * ============================================
     */

      /**
     * Insere o documento na tabela temporária.
     */
    // private function inserirDocumentoTemporario($nrseqorcamento, $nrtipoorcamento, $txDocument, $nrorg)
    // {
    //     $params = array(
    //         "NRSEQORCAMENTO"  => $nrseqorcamento,
    //         "NRTIPOORCAMENTO" => $nrtipoorcamento,
    //         "TXDOCUMENTO"     => $txDocument,
    //         "NRORG"           => $nrorg
    //     );
        
    //     $this->documentoFactory->inserirDocumentoTemporario($params);
    // }
    
    /**
     * Salva o relatório do documento da proposta na pasta "/tmp" e retorna seu caminho completo.
     */
    public function salvarRelatorio($nomeArquivo, $reportUrl)
    {
        try {
            $report = file_get_contents($reportUrl);
        } catch (\Exception $e) {
            return false;
        }

        $ds = DIRECTORY_SEPARATOR;

        if (is_dir($this->tempDir)) {
            $reportPath = $this->tempDir . $ds . "documento_cda" . $nomeArquivo . ".pdf";
        } else {
            $reportPath = __DIR__ . $ds . "documento_cda" . $nomeArquivo . ".pdf";
        }
        file_put_contents($reportPath, $report);
        return $reportPath;
    }

/**
     * Exclui os documentos temporários a ser utilizado para gerar o relatório.
     */
    // public function excluirDocumentosTemporarios($nrseqorcamento, $nrorg)
    // {
    //     $params = array(
    //         "NRSEQORCAMENTO" => $nrseqorcamento,
    //         "NRORG"          => $nrorg
    //     );
            
    //     $this->documentoFactory->excluirDocumentosTemporarios($params);
    // }
    

    /**
     * Baixa um arquivo
    **/
    // public static function downloadArquivo($fileDir, $fileName, $fileType, $fileSize, $istemporary = true)
    // {
    //     header('Accept-Ranges: none');
    //     header('Content-Type: ' . $fileType);
    //     header('Content-Length: ' . $fileSize);
    //     header('Content-Disposition: inline; filename="' . $fileName . '"');
        
    //     ob_clean();
    //     flush();
    //     readfile($fileDir);
        
    //     if ($istemporary) {
    //         unlink($fileDir);
    //     }
        
    //     exit;
    // }
    
    /**
     * Busca um documento no fileServer e retorna seu conteúdo.
    **/
    public function getDocumentoFileServer($urlArquivo, $nrorg)
    {
        $header = array (
            "apiKey: " . $this->fileServerApiKeys[$nrorg]
        );
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $urlArquivo);
        curl_setopt($ch, CURLOPT_POST, false);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 0);
        curl_setopt($ch, CURLOPT_TIMEOUT, 90);

        $resultado = curl_exec($ch);
        $error     = curl_error($ch);
        curl_close($ch);

        if (!empty($error)) {
            throw new \Exception($error);
        }

        if (strpos($resultado, "404 Not Found") !== false) {
            //throw new \Exception($resultado);
            return false;
        } else {
            return $resultado;
        }
    }
    
    /**
     * Busca um documento no fileServer e salva-o numa pasta temporária. Retorna o caminho físico do arquivo salvo.
    **/
    public function downloadDocFileServer($urlArquivo, $nomeArquivoParaSalvar, $nrorg)
    {
        $conteudoArquivo = $this->getDocumentoFileServer($urlArquivo, $nrorg);
    
        if ($conteudoArquivo !== false) {
            if (is_dir($this->tempDirFS) && is_writable($this->tempDirFS)) {
                $caminhoFisicoArquivo = $this->tempDirFS . DIRECTORY_SEPARATOR . $nomeArquivoParaSalvar;
                file_put_contents($caminhoFisicoArquivo, $conteudoArquivo);
                return $caminhoFisicoArquivo;
            } else {
                throw new \Exception("Falha ao salvar o documento. <br/>O diretório \"".$this->tempDirFS."\" não existe ou não possui permissão para escrita.");
            }
        } else {
            throw new \Exception("Falha ao baixar o documento. <br/>O arquivo correspondente à URL \"" . $urlArquivo . "\" não existe.");
        }
    }

    private function gerarDocumentoParteDeParticipantes($cda, $nrorg, $documento)
    {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[IND]" no documento inteiro
        $posicaoInd = strpos($documento, "[IND]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[IND]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
            $posicaoFechaLN = strpos($documento, "[/LN]", $posicaoInd);
            
            //posicao do primeiro "[LN]" que antecede o "[IND]"
            $posicaoPrimeiroLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoFechaLN) * -1));
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                        $result = "C";
                        $posIni = $posicaoPrimeiroTr;
                        $posFin = $posicaoFechaTr;
                        $tagFecha = "</w:tr>";
                } else {
                    $result = "L";
                
                    //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "[/LN]", $posicaoInd);
                    
                    //posicao do primeiro "[LN]" que antecede o "[IND]"
                    $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "[/LN]";
                }
            } elseif ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
            } else {
                $result = "L";
                
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "[/LN]", $posicaoInd);
                
                //posicao do primeiro "[LN]" que antecede o "[IND]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            $linhasCodigoItens = $this->getCodigoItensParticipantes($cda, $nrorg);
            $qtdItens = count($linhasCodigoItens);
            
            foreach ($linhasCodigoItens as $linhaCodigoItem) {
                //coloca o índice nos mnemônicos dos itens
                $clobLinhaSubstituida = $this->colocarIndiceMnemonicoParticip($linhaCodigoItem['IND'], $clobLinha);
                
                if ($result == "C") {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tr>';
                } else {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1;
                }
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosParticip($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }

    private function getCodigoItensParticipantes($parametros, $nrorg)
    {
        return $this->documentoFactory->getCodigoItensParticipantes(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
            "NRORG"               => $nrorg,
        ));
    }

    /**
     * Coloca o índice nos mnemônicos dos itens da proposta.
     */
    private function colocarIndiceMnemonicoParticip($codigoItem, $clobLinha)
    {
        $clobLinhaSubstituida = str_replace("[IND]", "[IND" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[PARTICIPANTE]", "[PARTICIPANTE" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[SETOR]", "[SETOR" . $codigoItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    private function retirarMnemonicosParticip($clobLinha)
    {
        $clobLinhaSubstituida = str_replace("[IND]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[PARTICIPANTE]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[SETOR]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    /**
     * Substitui os dados dos itens da proposta no documento.
     */
    private function substituirDadosParticipantes($particip, $documento)
    {
        $documento = str_replace("[IND".$particip["IND"]."]", "", $documento);
        
        $documento = str_replace("[PARTICIPANTE".$particip["IND"]."]", $particip["DSNOMEPARTICIP"], $documento);
        $documento = str_replace("[SETOR".$particip["IND"]."]", $particip["DSSETORPARTICIP"], $documento);
        
        return $documento;
    }
}
