import { Utilities } from '@/util/Utilities';
import {
    Form, LoadingService, Modal, ModalService, Select,
} from '@zeedhi/common';
import {
    Http, IDictionary, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class FilterCdaController {
    private ModalFiltrar!: Modal;

    public currentRow: any = {};

    public grupoOper: boolean = true;

    public eventFilter({ event }: any) {
        event?.preventDefault();
        setTimeout(() => {
            this.abrirModalFilter();
        }, 500);
    }

    public async criarModal() {
        this.ModalFiltrar = await ModalService.createFromJson('sm_cda/Modal/sm_cda_filter_modal', true);
        // this.permissaoOperadorCda();
    }

    public destruirModal() {
        this.ModalFiltrar?.destroy();
    }

    public abrirModalFilter() {
        this.ModalFiltrar.show();
        setTimeout(() => {
            this.insereFuncionarioLogado();
        }, 2000);
    }

    public fecharModalFilter() {
        this.ModalFiltrar.hide();
    }

    public operadorLogado: string = '';

    // Retornar true apenas para os usuários que tem IDAUTGRUPOP = 'ACSA' E 'DPAP'
    public async permissaoOperadorCda() {
        let grupo: boolean = false;
        try {
            const response = await Http.get('/getPermissaoOperadorCda');
            if (response.data.length > 0) {
                // Verifique se pelo menos um objeto tem IDAUTGRUPOP igual a "ACSA" ou "DPAP"
                grupo = !response.data.some((elemento: any) => elemento.IDAUTGRUPOP === 'ACSA' || elemento.IDAUTGRUPOP === 'DPAP');
            } else {
                grupo = true;
            }
        } catch (error) {
            Utilities.showErrorMessage(error);
        }
        return grupo;
    }
    
    public async getOperadorLogado() {
        let cdoper: string = '';
        try {
            const response = await Http.get('/getOperadorLogado');
            cdoper = response.data.CDOPERADOR;
        } catch (error) {
            Utilities.showErrorMessage(error);
        }
        return cdoper;
    }
    
    public async insereFuncionarioLogado() {
        LoadingService.show();
        const operador = Metadata.getInstance<Select>('formFilterCda-NMOPERADOR');
        operador.setValue(await this.getOperadorLogado());
        operador.disabled = await this.permissaoOperadorCda();
        LoadingService.hide();
    }

    public async insereUltimoPeriodo() {
        await Http.get('/getPeriodoApuraPonto').then((response: IDictionary) => {
            Metadata.getInstance<Select>('formFilterCda-PERIODOAPURAPONTO').value = response.data.data[0].DISPLAY;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public filterGrid() {
        if (Utilities.validateForm(Metadata.getInstance<Form>('form-filtrar-cda'))) {
            const operador = Metadata.getInstance<Select>('formFilterCda-NMOPERADOR');
            const grid = Metadata.getInstance<TekGrid>('cda-tek-grid');
            const periodo = Metadata.getInstance<Select>('formFilterCda-PERIODOAPURAPONTO');
            // const statusPeriodo = periodo.selectValue.STATUS;
            const regex = /(\d{2}\/\d{2}\/\d{4})/g;
            const [dtinicialperiodo, dtfinalperiodo] = periodo.value.match(regex) || [];

            grid.datasource.setFilter({
                NMOPERADOR: operador.selectValue.NMOPERADOR,
                CDOPERADOR: operador.selectValue.CDOPERADOR,
                CDFUNC: operador.selectValue.CDFUNC,
                CDEMPRESA: operador.selectValue.CDEMPRESA,
                CDFILIAL: (operador.selectValue.CDFILIAL === null) ? 'null' : operador.selectValue.CDFILIAL,
                NRORG: operador.selectValue.NRORG,
                DTINICIALPERIODO: dtinicialperiodo,
                DTFINALPERIODO: dtfinalperiodo,
            });
            grid.title = `Controle Diário de Atividades - ${operador.selectValue.NMOPERADOR}`;
            this.fecharModalFilter();
        }
    }
}
